/*
 * itg-irqupdater.cpp
 *
 *  Created on: Oct 27, 2010
 *      Author: root
 */

#include "../Includes/qstdinc.h"
#include "../Includes/qlogger.h"
#include "../Includes/qaffinity.h"

#ifndef _WIN32
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <sys/syscall.h>
#include <ftw.h>
#endif

QLOG_REGISTER_SOURCE

static volatile int g_nCPUCount = -1;
int QAF_GetCPUCount()
{
	if (g_nCPUCount > 0) return g_nCPUCount;

	int nCPUCount = 0;
#ifndef _WIN32
	FILE* pFile = fopen("/proc/cpuinfo", "r+t");
	if (!pFile)
	{
		QLogInfo(0, "[UTIL] Error detecting CPU count");
		exit(0);
	}

	char sBuff[1024];
	while (!feof(pFile))
	{
		if (fgets(sBuff, 1024, pFile) == sBuff)
		{
			if (strstr(sBuff, "processor"))
			{
				nCPUCount++;
			}
		}
	}
	fclose(pFile);
#else
	SYSTEM_INFO si;
	GetSystemInfo(&si);
	nCPUCount = si.dwNumberOfProcessors; 
#endif
	g_nCPUCount = nCPUCount;
	return nCPUCount;
}

bool QAF_UpdateMask(uint32_t nMask)
{
#ifndef _WIN32
	/* Update affinity */
	cpu_set_t cpu_mask;
	CPU_ZERO(&cpu_mask);
	int nCPUCount = QAF_GetCPUCount();
	for (int i=0; i<nCPUCount; i++)
	{
		if (nMask & (1 << i))
		{
			CPU_SET(i, &cpu_mask);
		}
	}

	int nRetVal = sched_setaffinity((pid_t)syscall(SYS_gettid), sizeof(cpu_mask), &cpu_mask);
	if (nRetVal)
	{
		QLogInfo(0, "[UTIL] Error in sched_setaffinity (%d)", errno);
		exit(0);
	}
#else
	if (SetThreadAffinityMask(GetCurrentThread(), nMask) == 0)
	{
		QLogInfo(0, "[UTIL] Error in SetThreadAffinityMask (%08x)(%d)", nMask, ::GetLastError());
		exit(0);
	}
#endif

	return true;
}
